if not (rawget(_G, "customhud")) return end

local modname = "sa1hud"
customhud.SetupItem("score", modname) -- By just setting this, it'll go away!
customhud.SetupItem("time", modname)
customhud.SetupItem("rings", modname)
customhud.SetupItem("lives", modname)
customhud.SetupItem("stagetitle", modname)
customhud.SetupItem("bossmeter", modname)

local clientplayer
local boss
local barlength = 0
local seglength = 12
local disphp = 0

local dontshow = "_DONTSHOWTHISBOSSMETER" -- hopefully no one makes a boss named this? :p
local cardlength = (2*TICRATE) + (NUMTRANSMAPS*2)

local bossnames = {
	-- Vanilla SRB2
	[MT_EGGMOBILE] = "Egg Mobile",
	[MT_EGGMOBILE2] = "Egg Slimer",
	[MT_EGGMOBILE3] = "Sea Egg",
	[MT_EGGMOBILE4] = "Egg Coliseum",
	[MT_FANG] = "Fang",
	[MT_METALSONIC_BATTLE] = "Metal Sonic",
	[MT_CYBRAKDEMON] = "Black Eggman",
	[MT_BLACKEGGMAN] = "Black Eggman",
}

local addonbosses = {
	-- SUGOI
	MT_SILVER_318 = "Silver",
	MT_TURRETMOBILE = "Turret Mobile",
	MT_PURPLEHEART = dontshow,
	-- SUBARASHII
	MT_EGGDOMINATOR = "Egg Dominator",
	MT_RRZBOSS = dontshow,
	MT_EGGWORLD = dontshow,
	MT_EXPSTAR = dontshow,
	-- KIMOKAWAIII
	MT_EGGGUNNER = "Egg Gunner",
	MT_BOUNCYBOI = "BoBo the Bouncy Boi",
	MT_CYBERDETON = "Cyber Deton",
	MT_POINTYEGG = "Pointy Egg",
	MT_LJSONIC = "LJ Sonic",
	MT_CONFLEGGRATOR = "Confleggrator",
	MT_AALPBOSS = "Egg Robo",
	MT_DARKSTABOT = "Dark Stabot",
	MT_EGGLAS = "Egg Lasershow",
	MT_PROTAGONIST = "True Protagonist",
	MT_GUNDAM = dontshow,
	MT_SOAES = dontshow,
	-- (potential) character bosses
	MT_SONIC = "Sonic",
	MT_TAILS = "Tails",
	MT_KNUCKLES = "Knuckles",
	MT_AMY = "Amy Rose",
	MT_SHADOW = "Shadow",
	MT_SILVER = "Silver",
	-- Misc levels
	MT_EGGMOBILE7 = "Egg Boiler", -- True Arena name, I'd like to know if FuriousFox had an official name for this
	MT_STRAYBOLTS_BOSS = "Stray-Bolts",
	MT_OLDK = "K.T.E.",
	MT_ANASTASIA = dontshow,
	MT_BOSSRIDE = "Player", -- this one never gets used, included anyway to tell the game you have it loaded
	-- True Arena's ports
	MT_SANDSUB_326 = "Egg Sand Sub",
	MT_GREENHILLBOSS = "Ball & Chain",
	MT_SUPERHOOD = "Super Robo-Hood",
	MT_INFINITE_318 = "Infinite",
}

local function setBoss(mo)
	if (mo.health <= 0) return end
	
	if not ((clientplayer and clientplayer.valid) -- without client player, we'd have to iterate everyone TWICE AAA :v
	and (clientplayer.mo and clientplayer.mo.valid))
		return
	end 
	
	if (P_CheckSight(mo, clientplayer.mo))
		if ((boss and boss.valid) and not (P_CheckSight(boss, clientplayer.mo))) -- boss exists, but not in line of sight?
		or not (boss and boss.valid) -- or the boss just doesn't exist
			boss = mo
		end
	end
end

local function bossHurt(mo, inf, src)
	if (mo.flags & MF_BOSS)
	and (clientplayer and clientplayer.valid)
	and (clientplayer.mo and clientplayer.mo.valid)
	and ((inf and inf.valid and inf == clientplayer.mo)
	or (src and src.valid and src == clientplayer.mo))
		boss = mo
	end
end

local function isMobjTypeValid(mt)
	if (pcall(do return _G[mt] end))
		return _G[mt]
	else
		return nil
	end
end

local function mapSet(mo)
	boss = nil
	barlength = 0
	disphp = 0

	for k,v in pairs(addonbosses)
		local mt = isMobjTypeValid(k)
		if not (mt) continue end
		bossnames[mt] = v
	end
end

local function stupidGarbage() -- Because some mods like to reset hud enables/disables every tic instead of once (Boss Mayhem... though I'm kinda guilty of it too ^^;)
	if (customhud.CheckType("score") == modname) hud.disable("score") end
	if (customhud.CheckType("time") == modname) hud.disable("time") end
	if (customhud.CheckType("rings") == modname) hud.disable("rings") end
	if (customhud.CheckType("lives") == modname) hud.disable("lives") end
	if (customhud.CheckType("stagetitle") == modname) hud.disable("stagetitle") end
end

addHook("BossThinker", setBoss)
addHook("MobjDamage", bossHurt)
addHook("MapChange", mapSet)
--addHook("MapLoad", mapSet)
addHook("ThinkFrame", function()
	stupidGarbage()
end)

local screenwidth = 320
local screenheight = 200

local function sadxTime(v, player)
	if (customhud.CheckType("time") != modname) return end
	if (customhud.CheckEnabled("time") != true) return end

	if not (player and player.valid) return end

	local startx = 8
	local starty = 12

	local label = v.cachePatch("SATIME")
	local colon = v.cachePatch("SACOLON")

	v.draw(startx, starty, label, V_SNAPTOLEFT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)

	local minutes = min(99, G_TicsToMinutes(player.realtime))
	customhud.CustomNum(v, startx+40, starty, minutes, "SANUM", 8, 2, V_SNAPTOLEFT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
	v.draw(startx+56, starty, colon, V_SNAPTOLEFT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
	customhud.CustomNum(v, startx+64, starty, G_TicsToSeconds(player.realtime), "SANUM", 8, 2, V_SNAPTOLEFT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
	v.draw(startx+80, starty, colon, V_SNAPTOLEFT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
	customhud.CustomNum(v, startx+88, starty, G_TicsToCentiseconds(player.realtime), "SANUM", 8, 2, V_SNAPTOLEFT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
end

local function sadxRings(v, player)
	if (customhud.CheckType("rings") != modname) return end
	if (customhud.CheckEnabled("rings") != true) return end

	if not (player and player.valid) return end

	local startx = 8
	local starty = 28

	local ring = v.cachePatch("SARING")

	v.draw(startx, starty-1, ring, V_SNAPTOLEFT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)

	if (player.rings <= 0)
		local origvalue = v.localTransFlag() >> V_ALPHASHIFT
		local length = NUMTRANSMAPS - origvalue
		local pulse = abs(((leveltime/2) % (NUMTRANSMAPS*2)) - NUMTRANSMAPS)
		local trans = 0

		if (origvalue >= NUMTRANSMAPS)
			return
		end

		if (origvalue > 0)
			pulse = ($1 * length) / NUMTRANSMAPS
			trans = (origvalue + pulse)
		end

		if (trans < NUMTRANSMAPS)
			trans = $1 << V_ALPHASHIFT
			customhud.CustomNum(v, startx+24, starty, player.rings, "SANUM", 8, 3, V_SNAPTOLEFT|V_SNAPTOTOP|V_PERPLAYER|trans)
		end

		trans = origvalue + (length - pulse)

		if (trans < NUMTRANSMAPS)
			trans = $1 << V_ALPHASHIFT
			customhud.CustomNum(v, startx+24, starty, player.rings, "SARNUM", 8, 3, V_SNAPTOLEFT|V_SNAPTOTOP|V_PERPLAYER|trans)
		end
	else
		customhud.CustomNum(v, startx+24, starty, player.rings, "SANUM", 8, 3, V_SNAPTOLEFT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
	end
end

local function sadxLives(v, player)
	if (customhud.CheckType("lives") != modname) return end
	if (customhud.CheckEnabled("lives") != true) return end
	
	if not (G_GametypeUsesLives()) return end
	if not ((player and player.valid) and (player.mo and player.mo.valid)) return end
	if not (skins[player.mo.skin] and skins[player.mo.skin].valid) return end
	
	local startx = 8
	local starty = screenheight-32

	local bg = v.cachePatch("SAFACEBG")
	local face = v.getSprite2Patch(player.mo.skin, SPR2_LIFE, (player.powers[pw_super] != 0), 0, 0)
	
	v.draw(startx-1, starty-1, bg, V_SNAPTOLEFT|V_SNAPTOBOTTOM|V_PERPLAYER|V_HUDTRANS)
	v.draw((startx+10), (starty+14), face, V_SNAPTOLEFT|V_SNAPTOBOTTOM|V_PERPLAYER|V_HUDTRANS, v.getColormap(player.mo.skin, player.mo.color))
	customhud.CustomNum(v, startx+24, starty+4, player.lives, "SANUM", 8, 2, V_SNAPTOLEFT|V_SNAPTOBOTTOM|V_PERPLAYER|V_HUDTRANS)
end

local function sadxBoss(v, player)
	if (customhud.CheckType("bossmeter") != modname) return end
	if (customhud.CheckEnabled("bossmeter") != true) return end

	if not (boss and boss.valid) return end
	if (bossnames[boss.type] == dontshow) return end

	local startx = screenwidth-17
	local starty = 21

	local bar = v.cachePatch("SABAR")
	local ends = v.cachePatch("SABAREND")

	local fills = {
		[1] = {
			{22, -5, 31}, -- height, offset, color
			{12, 0, 19},
			{3, -5, 0},
			{3, 13, 0},
			{4, 1, 17},
			{2, 1, 15},
			{2, 9, 21},
			{1, 12, 21}
		},
		[2] = {
			{12, 0, 134},
			{4, 1, 132},
			{2, 1, 130},
			{2, 9, 135},
			{1, 12, 135}
		},
	}

	local name = "Target:"
	if (bossnames[boss.type])
		name = bossnames[boss.type]..":"

		if (isMobjTypeValid("MT_BOSSRIDE"))
			if (_G["MT_BOSSRIDE"] == boss.type)
			and ((boss.rider and boss.rider.valid)
			and (boss.rider.player and boss.rider.player.valid))
				name = boss.rider.player.name..":"
			end
		end
	end

	if (player == clientplayer)
		if (barlength) and ((boss.health <= 0) or (boss == nil))
			barlength = $1-1

			if (barlength <= 0)
				boss = nil
				barlength = 0
				return
			end
		else
			local maxhealth = boss.info.spawnhealth
			seglength = 12

			if (isMobjTypeValid("MT_OLDK"))
				if (_G["MT_OLDK"] == boss.type)
					seglength = $1 * 8
					maxhealth = 1
				end
			end

			barlength = maxhealth*seglength

			while ((barlength > 168) and (seglength > 1))
				seglength = $1-1
				barlength = maxhealth*seglength
			end
		end

		local wantedhp = min(barlength, boss.health*seglength)

		if (boss.health <= 0)
			wantedhp = 0
			barlength = $1-1

			if (barlength <= 0)
				boss = nil
				barlength = 0
				return
			end
		end

		if (disphp != wantedhp)
			local dir = -1
			if (disphp < wantedhp)
				dir = 1
			end

			disphp = $1+dir
		end

		if (disphp > barlength)
			disphp = barlength
		end
	end

	local leftx = startx-(barlength-1)-16

	local masterfill = fills[1]
	for i = 1,#masterfill
		local curfill = masterfill[i]
		v.drawFill(leftx+8, starty+curfill[2], barlength, curfill[1], curfill[3]|V_SNAPTOTOP|V_SNAPTORIGHT|V_PERPLAYER)
	end

	if (disphp > 0)
		masterfill = fills[2]
		for k = 1,#masterfill
			local curfill = masterfill[k]
			local aa = disphp+1

			v.drawFill(startx-8-disphp, starty+curfill[2], aa, curfill[1], curfill[3]|V_SNAPTOTOP|V_SNAPTORIGHT|V_PERPLAYER)
		end
	end

	v.draw(startx, starty, ends, V_SNAPTOTOP|V_SNAPTORIGHT|V_FLIP|V_PERPLAYER)
	v.draw(leftx, starty, ends, V_SNAPTOTOP|V_SNAPTORIGHT|V_PERPLAYER)

	local namex = (startx - v.stringWidth(name, V_ALLOWLOWERCASE))
	if (leftx < namex)
		namex = leftx
	end
	v.drawString(namex, starty-13, name, V_ALLOWLOWERCASE|V_SNAPTOTOP|V_SNAPTORIGHT|V_PERPLAYER)
end

local function sadxTitle(v, player, tick)
	if (customhud.CheckType("stagetitle") != modname) return end
	if (customhud.CheckEnabled("stagetitle") != true) return end

	if not (mapheaderinfo[gamemap].lvlttl) return end
	if (mapheaderinfo[gamemap].levelflags & LF_NOTITLECARD) return end
	if (tick < 0 or tick > cardlength) return end

	local startx = screenwidth/2
	local starty = 3*screenheight/4

	local cardnum = 1
	
	local color = SKINCOLOR_BLUE
	if (player and player.valid)
		color = player.skincolor
	end
	
	if (color == SKINCOLOR_YOGURT
	or color == SKINCOLOR_TAN
	or color == SKINCOLOR_PEACHY
	or color == SKINCOLOR_SUNSET
	or color == SKINCOLOR_APRICOT
	or color == SKINCOLOR_ORANGE
	or color == SKINCOLOR_GOLD
	or color == SKINCOLOR_YELLOW)
		-- skincolors on the light orange/yellow ends of the spectrum get inverted shading!
		-- a tiny detail from sa1 tails that no one except me noticed!
		cardnum = $1+2
	end
	
	local trans = 0
	if (tick < NUMTRANSMAPS)
		trans = (NUMTRANSMAPS-tick) << V_ALPHASHIFT
	elseif (tick > cardlength-NUMTRANSMAPS)
		trans = (tick - (cardlength-NUMTRANSMAPS)) << V_ALPHASHIFT
	end
	
	if not (mapheaderinfo[gamemap].actnum)
		cardnum = $1+1
	end
	
	local sub = "ACTION STAGE"

	if (#(mapheaderinfo[gamemap].subttl) > 0)
		sub = string.upper(mapheaderinfo[gamemap].subttl)
	elseif (mapheaderinfo[gamemap].levelflags & LF_WARNINGTITLE)
		sub = "BOSS STAGE"
	elseif (G_IsSpecialStage(gamemap))
		sub = "SPECIAL STAGE"
	elseif (maptol & TOL_NIGHTS)
		sub = "NiGHTS STAGE"
	elseif (circuitmap)
		sub = "CIRCUIT STAGE"
	elseif (gametype >= GT_MATCH) -- not G_RingslingerGametype, because that would show up in SP stages if ringslinger is on
		sub = "RINGSLINGER STAGE"
	elseif (rawget(_G, "sugoi") and sugoi.IsHubLevel())
		sub = "ADVENTURE FIELD"
	end
	
	local card = v.cachePatch("SATCARD"..cardnum)
	
	v.draw(startx, starty, card, V_PERPLAYER|trans, v.getColormap(TC_DEFAULT, color))
	
	if (mapheaderinfo[gamemap].actnum)
		customhud.CustomNum(v, startx-88, starty-8, mapheaderinfo[gamemap].actnum, "SALNUM", nil, nil, V_PERPLAYER|trans, "center")
		
		local ttllength = customhud.CustomFontStringWidth(v, mapheaderinfo[gamemap].lvlttl, "SAFNT", 4)
		if (ttllength > 168)
			if (ttllength > 212)
				local diff = ttllength - (screenwidth-108)
				customhud.CustomFontString(v, startx-52-diff, starty-10, mapheaderinfo[gamemap].lvlttl, "SAFNT", 4, nil, V_PERPLAYER|trans, "left")
			else
				customhud.CustomFontString(v, startx-52, starty-10, mapheaderinfo[gamemap].lvlttl, "SAFNT", 4, nil, V_PERPLAYER|trans, "left")
			end
		else
			customhud.CustomFontString(v, startx+32, starty-10, mapheaderinfo[gamemap].lvlttl, "SAFNT", 4, nil, V_PERPLAYER|trans, "center")
		end
		
		local sublength = customhud.CustomFontStringWidth(v, sub, "SASFT", 6, 6)
		if (sublength > 120)
			if (sublength > 214)
				local diff = sublength - (screenwidth-106)
				customhud.CustomFontString(v, startx-54-diff, starty+6, sub, "SASFT", 6, 6, V_PERPLAYER|trans, "left")
			else
				customhud.CustomFontString(v, startx-54, starty+6, sub, "SASFT", 6, 6, V_PERPLAYER|trans, "left")
			end
		else
			customhud.CustomFontString(v, startx+5, starty+6, sub, "SASFT", 6, 6, V_PERPLAYER|trans, "center")
		end
	else
		customhud.CustomFontString(v, startx, starty-10, mapheaderinfo[gamemap].lvlttl, "SAFNT", 4, nil, V_PERPLAYER|trans, "center")
		customhud.CustomFontString(v, startx, starty+6, sub, "SASFT", 6, nil, V_PERPLAYER|trans, "center")
	end
end

hud.add(function(v, player)
	stupidGarbage()
	if (player.bot) return end

	if not (splitscreen and player != players[0])
		clientplayer = player
	end

	if not (maptol & TOL_NIGHTS)
		sadxTime(v, player)
		sadxRings(v, player)
		sadxLives(v, player)
		sadxBoss(v, player)
	end
end, "game")

hud.add(function(v, player, tick)
	if (player.bot) return end
	if (tick > cardlength) return end
	sadxTitle(v, player, tick)
end, "titlecard")
